/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleshellgeneticalgorithm.h"
#include "graleshellinstance.h"
#include <shellp/iosystem.h>
#include <grale/lensinversiongenomebase.h>
#include <grale/gravitationallens.h>
#include <stdio.h>

#include "debugnew.h"

using namespace shellp;
using namespace mogal;

GraleShellGeneticAlgorithm::GraleShellGeneticAlgorithm()
{
	saveintermediate = false;
	count = 0;
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
        iosys = gsinst->getIOSystem();
}

GraleShellGeneticAlgorithm::GraleShellGeneticAlgorithm(const std::string &pref)
{
	saveintermediate = true;
	prefix = pref;
	count = 0;
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
        iosys = gsinst->getIOSystem();
}

GraleShellGeneticAlgorithm::~GraleShellGeneticAlgorithm()
{
}

void GraleShellGeneticAlgorithm::feedbackStatus(const std::string &str) const
{
	iosys->writeOutputLine(str);
}

void GraleShellGeneticAlgorithm::onCurrentBest(const std::list<mogal::Genome *> &bestgenomes) const
{
	std::list<mogal::Genome *>::const_iterator it;
	std::string line("Current best: ");
	
	for (it = bestgenomes.begin() ; it != bestgenomes.end() ; it++)
	{
		char buf[256];
		sprintf(buf, "( %s)", (*it)->getFitnessDescription().c_str());
		line += std::string(buf);
	}
	iosys->writeOutputLine(line);
	
	if (saveintermediate)
	{
		char buf[256];
		int num = 1;

		count++;

		for (it = bestgenomes.begin() ; it != bestgenomes.end() ; it++,num++)
		{
			sprintf(buf, "_%05d_%02d.lensdata", count, num);
			std::string fname = prefix+std::string(buf);

			const grale::LensInversionGenomeBase *genome = (const grale::LensInversionGenomeBase *)(*it);
			double totalmass;
			std::string errstr;
			
			grale::GravitationalLens *lens = genome->createLens(&totalmass,errstr);
		
			if (!lens)
				iosys->writeOutputLine(std::string("WARNING: Couldn't create lens for intermediate solution: ") + errstr);
			else
			{
				if (!lens->save(fname))
					iosys->writeOutputLine(std::string("WARNING: Couldn't save intermediate solution as ") + fname + std::string(": ") + lens->getErrorString());
			}
			delete lens;
		}
	}
}

bool GraleShellGeneticAlgorithm::stopRemoteAlgorithm()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();

	if (pGSInst->isInterrupted())
		return true;
	return false;
}

bool GraleShellGeneticAlgorithm::onAlgorithmLoop(GAFactory &factory, bool generationInfoChanged)
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();

	if (pGSInst->isInterrupted())
	{
		setErrorString("Calculation interrupted by user");
		return false;
	}

	return GeneticAlgorithm::onAlgorithmLoop(factory, generationInfoChanged);
}

